<?php
/**
 * Record Controller
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2010.
 * Copyright (C) Leipzig University Library 2016.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Controller
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
namespace fid_bbi\Controller;

use finc\Mailer\Mailer;
use VuFind\Exception\Mail as MailException;
use Zend\Log\LoggerAwareInterface as LoggerAwareInterface;
use Zend\Mail\Address;
use Zend\Validator\NotEmpty;
use Zend\Validator\StringLength;

/**
 * Record Controller
 *
 * @category VuFind
 * @package  Controller
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
class RecordController extends \fid\Controller\RecordController implements
    LoggerAwareInterface
{
    use \VuFind\Log\LoggerAwareTrait;

    /**
     * ReportErrors action - controller method
     *
     * @return \Zend\View\Model\ViewModel
     */
    public function reportErrorsAction()
    {
        // Force login if necessary:
        $config = $this->getConfig();
        if ((!isset($config->Mail->require_login) || $config->Mail->require_login)
            && !$this->getUser()
        ) {
            return $this->forceLogin();
        }

        $params['email'] = null;
        /*
         * refs #15163: remove user data
         */
        if ($user = $this->getUser()) {
            $params['email'] = trim($user->email);
        }

        // Create view
        $view = $this->createReportErrorsEmailViewModel($params);

        // Set up reCaptcha
        //todo: testen!
        $view->useRecaptcha = $this->recaptcha()->active('reportErrors');

        // Process form submission
        if ($this->formWasSubmitted('submit', $view->useRecaptcha)) {
            // Collect the data submitted by form
            $params['reply_requested'] = $view->reply_requested;
            $params['email']  = !empty($view->email) ? $view->email : '';
            /*
             * refs #15163: remove user data
             */
//            if ($view->reply_requested) {
//                if ($user) {
//                    $params['firstname'] = trim($user->firstname);
//                    $params['lastname'] = trim($user->lastname);
//                }
//            }
            $params['comment'] = !empty($view->comment) ? $view->comment : '';
            $params['subject'] = !empty($view->subject) ? $view->subject : '';
            $params['eula'] = isset($view->eula) ? $view->eula : '';

            // Validate data submitted by form
            $validatorStrLength = new StringLength(['min' => 10]);
            $validatorNotEmpty = new NotEmpty();

            if (!$validatorStrLength->isValid($params['comment'])) {
                $this->flashMessenger()
                    ->addMessage('report_errors_comment_blank', 'error');
            } elseif (!$validatorNotEmpty->isValid($params['eula'])) {
                $this->flashMessenger()
                    ->addMessage('fid::error_eula_accepted', 'error');
            }else {
                // All params are valid, set timestamp for current params set
                $params['timestamp'] = date('d.m.Y H:i');

                // Attempt to send the email and show an appropriate flash message:
                try {
                    $this->sendReportErrorsEmail($params);
                    $this->flashMessenger()->addMessage('report_errors_send_success', 'success');
                    return $this->redirectToRecord();
                } catch (MailException $e) {
                    $this->flashMessenger()->addMessage($e->getMessage(), 'error');
                }
            }
        }

        // Display the template:
        $view->setTemplate('record/reporterrorsform');
        return $view;
    }

    /**
     * Acquisition action - controller method
     *
     * @return \Zend\View\Model\ViewModel
     */
    public function acquisitionAction()
    {
        // Force login if necessary:
        $config = $this->getConfig();
        if ((!isset($config->Mail->require_login) || $config->Mail->require_login)
            && !$this->getUser()
        ) {
            return $this->forceLogin();
        }

        $params['email'] = null;
        if ($user = $this->getUser()) {
            $params['email'] = trim($user->email);
        }

        // Create view
        $view = $this->createAcquisitionEmailViewModel($params);

        // Set up reCaptcha
        //todo: testen!
        $view->useRecaptcha = $this->recaptcha()->active('acquisition');

        // Process form submission
        if ($this->formWasSubmitted('submit', $view->useRecaptcha)) {
            // Collect the data submitted by form
            $params['email']  = !empty($view->email) ? $view->email : '';
            $params['comment'] = !empty($view->comment) ? $view->comment : '';
            $params['subject'] = !empty($view->subject) ? $view->subject : '';
            $params['eula'] = isset($view->eula) ? $view->eula : '';

            // Validate data submitted by form
            $validatorNotEmpty = new NotEmpty();

            if (!$validatorNotEmpty->isValid($params['eula'])) {
                $this->flashMessenger()
                    ->addMessage('fid::error_eula_accepted', 'error');
            }else {
                // All params are valid, set timestamp for current params set
                $params['timestamp'] = date('d.m.Y H:i');

                // Attempt to send the email and show an appropriate flash message:
                try {
                    $this->sendAcquisitionEmail($params);
                    $this->flashMessenger()->addMessage('report_errors_send_success', 'success');
                    return $this->redirectToRecord();
                } catch (MailException $e) {
                    $this->flashMessenger()->addMessage($e->getMessage(), 'error');
                }
            }
        }

        // Display the template:
        $view->setTemplate('record/acquisitionform');
        return $view;
    }

    /**
     * Create a new ViewModel to use as a PDA-Email form.
     *
     * @param array  $params         Parameters to pass to ViewModel constructor.
     *
     * @return ViewModel
     */
    protected function createReportErrorsEmailViewModel($params = null)
    {
        // Build view:
        $view = $this->createViewModel($params);

        // Send parameters back to view so form can be re-populated:
        if ($this->getRequest()->isPost()) {
            $view->email = $this->params()->fromPost('reporterrors_email');
            $view->reply_requested = ($this->params()->fromPost('reporterrors_checkbox') ? true : false);
            $view->comment = $this->params()->fromPost('comment');
            $view->subject = $this->params()->fromPost('reporterrors_subject');
            $view->eula = $this->params()->fromPost('reporterrors_checkbox_eula');
        }

        // Collect the records metadata
        $keyMethodMapper = [
            'id'            => 'getUniqueID',
            'author'        => 'getDeduplicatedAuthors',
            'title'         => 'getTitle',
            'title_short'   => 'getShortTitle',
            'year'          => 'getPublishDateSort',
            'isbn'          => 'getISBNs'
        ];
        $driver = $this->loadRecord();
        foreach ($keyMethodMapper as $var => $method) {
            $view->$var = $driver->tryMethod($method);
        }

        $view->author = !empty($view->author) ?
            $this->getAuthors($view->author) : null;

        return $view;
    }

    /**
     * Create a new ViewModel to use as a Acquisition-Email form.
     *
     * @param array  $params         Parameters to pass to ViewModel constructor.
     *
     * @return ViewModel
     */
    protected function createAcquisitionEmailViewModel($params = null)
    {
        // Build view:
        $view = $this->createViewModel($params);

        // Send parameters back to view so form can be re-populated:
        if ($this->getRequest()->isPost()) {
            $view->email = $this->params()->fromPost('acquisition_email');
            $view->comment = $this->params()->fromPost('acquisition_comment');
            $view->subject = $this->params()->fromPost('acquisition_subject');
            $view->eula = $this->params()->fromPost('acquisition_checkbox_eula');
        }

        // Collect the records metadata
        $keyMethodMapper = [
            'id'            => 'getUniqueID',
            'author'        => 'getDeduplicatedAuthors',
            'title'         => 'getTitle',
            'title_short'   => 'getShortTitle',
            'isbn'          => 'getISBNs'
        ];
        $driver = $this->loadRecord();
        foreach ($keyMethodMapper as $var => $method) {
            $view->$var = $driver->tryMethod($method);
        }

        $view->author = !empty($view->author) ?
            $this->getAuthors($view->author) : null;

        return $view;
    }

    /**
     * Formatted author data
     *
     * @param array $data author data
     *
     * @return array
     */
    protected function getAuthors($data)
    {
        $formattedAuthors = [];
        $types = [
            'main',
            'secondary',
            'corporate',
            'corporate_secondary'
        ];

        foreach ($types as $type) {
            if (!empty($data[$type])) {
                foreach ($data[$type] as $key => $value) {
                    $formattedAuthors[] = $key;
                }
            }
        }
        return $formattedAuthors;
    }

    /**
     * Send PDA order via e-mail.
     *
     * @param array $params Data to be used for Email template
     *
     * @return void
     * @throws MailException
     */
    protected function sendReportErrorsEmail($params)
    {
        $emailProfile = $this->getEmailProfile('ReportErrors');
        $renderer = $this->getViewRenderer();

        // Collect the records metadata
        $keyMethodMapper = [
            'id'            => 'getUniqueID',
            'author'        => 'getDeduplicatedAuthors',
            'title'         => 'getTitle',
            'title_short'   => 'getShortTitle',
            'year'          => 'getPublishDateSort',
            'isbn'          => 'getISBNs'
        ];
        $driver = $this->loadRecord();
        foreach ($keyMethodMapper as $var => $method) {
            $params[$var] = $driver->tryMethod($method);
        }
        $params['author'] = !empty($params['author']) ?
            $this->getAuthors($params['author']) : null;
        $params['driver'] = $driver;

        // Custom template for emails (html-only)
        $bodyHtml = $renderer->render(
            'Email/reporterrors-html.phtml',
            $params
        );
        // Custom template for emails (text-only)
        $bodyPlain = $renderer->render(
            'Email/reporterrors-plain.phtml',
            $params
        );

        // Build the subject
        $subject = (isset($emailProfile->subject))
            ? sprintf(
                $emailProfile->subject,
                $params['id'],
                $params['subject']
            ) : $this->translate('ReportErrors');

        // Set reply address and name if available
        if ($params['reply_requested'] == true) {
            $replyTo = !empty($params['email']) ? $params['email'] : $emailProfile->from;
//            $replyToName
//                = (isset($params['firstname']) && isset($params['lastname'])) && !empty($replyTo)
//                ? $params['firstname'] . ' ' . $params['lastname']
//                : '';
//            $reply = new Address($replyTo, $replyToName);
            $reply = new Address($replyTo);
        }

        // Get mailer
        $mailer = new Mailer(
            $this->serviceLocator->get('VuFind\Mailer')->getTransport()
        );

        // Send the email
        $mailer->sendTextHtml(
            new Address($emailProfile->to),
            new Address($emailProfile->from),
            isset($reply) ? $reply : $emailProfile->from,
            $subject,
            $bodyHtml,
            $bodyPlain
        );
    }

    /**
     * Send Acquisition via e-mail.
     *
     * @param array $params Data to be used for Email template
     *
     * @return void
     * @throws MailException
     */
    protected function sendAcquisitionEmail($params)
    {
        $emailProfile = $this->getEmailProfile('Acquisition');
        $renderer = $this->getViewRenderer();

        // Collect the records metadata
        $keyMethodMapper = [
            'id'            => 'getUniqueID',
            'author'        => 'getDeduplicatedAuthors',
            'title'         => 'getTitle',
            'title_short'   => 'getShortTitle',
            'isbn'          => 'getISBNs'
        ];
        $driver = $this->loadRecord();
        foreach ($keyMethodMapper as $var => $method) {
            $params[$var] = $driver->tryMethod($method);
        }
        $params['author'] = !empty($params['author']) ?
            $this->getAuthors($params['author']) : null;
        $params['driver'] = $driver;

        // Custom template for emails (html-only)
        $bodyHtml = $renderer->render(
            'Email/acquisition-html.phtml',
            $params
        );
        // Custom template for emails (text-only)
        $bodyPlain = $renderer->render(
            'Email/acquisition-plain.phtml',
            $params
        );

        // Build the subject
        $subject = (isset($emailProfile->subject))
            ? sprintf(
                $emailProfile->subject,
                $params['title'],
                $params['subject']
            ) : $this->translate('Book Suggestion');

        // Set reply address and name if available
        $replyTo = !empty($params['email']) ? $params['email'] : $emailProfile->from;
        $reply = new Address($replyTo);

        // Get mailer
        $mailer = new Mailer(
            $this->serviceLocator->get('VuFind\Mailer')->getTransport()
        );

        // Send the email
        $mailer->sendTextHtml(
            new Address($emailProfile->to),
            new Address($emailProfile->from),
            isset($reply) ? $reply : $emailProfile->from,
            $subject,
            $bodyHtml,
            $bodyPlain
        );
    }
}
