<?php
/**
 * Table Definition for resource_tags
 *
 * PHP version 7
 *
 * Copyright (C) Villanova University 2010.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Db_Table
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
namespace fid_bbi\Db\Table;

use VuFind\Db\Table\ResourceTags as BaseTable;
use Zend\Db\Adapter\Adapter;
use Zend\Db\Sql\Expression;
use Zend\Db\Sql\Sql;

/**
 * Table Definition for resource_tags
 *
 * @category VuFind
 * @package  Db_Table
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
class ResourceTags extends BaseTable
{
    use ResourceTagsTrait;

    /**
     * Get Resource Tags
     *
     * @param string $userId     ID of user
     * @param string $resourceId ID of the resource
     * @param string $tagId      ID of the tag
     * @param string $order      The order in which to return the data
     * @param string $page       The page number to select
     * @param string $limit      The number of items to fetch
     *
     * @return \Zend\Paginator\Paginator
     */
    public function getResourceTags(
        $userId = null, $resourceId = null, $tagId = null,
        $order = null, $page = null, $limit = 20
    ) {
        $order = (null !== $order)
            ? [$order]
            : ["username", "tag", "title"];

        $sql = $this->getSql();
        $select = $sql->select();
        $select = $select->columns([
            '*',
            'first_posted' => new Expression(
                'MIN(?)',['posted'],
                [Expression::TYPE_IDENTIFIER]
            )
        ]);
        $select->join(
            ['t' => 'tags'],
            'resource_tags.tag_id = t.id',
            [
                'tag' =>
                    $this->caseSensitive ? 'tag' : new Expression('lower(tag)')
            ]
        );
        $select->join(
            ['u' => 'user'],
            'resource_tags.user_id = u.id',
            ["username" => "username"]
        );
        $select->join(
            ['r' => 'resource'],
            'resource_tags.resource_id = r.id',
            ["title" => "title"]
        );
        if (null !== $userId) {
            $select->where->equalTo('resource_tags.user_id', $userId);
        }
        if (null !== $resourceId) {
            $select->where->equalTo('resource_tags.resource_id', $resourceId);
        }
        if (null !== $tagId) {
            $select->where->equalTo('resource_tags.tag_id', $tagId);
        }
        $select->order($this->formatTagOrder($order));

        if (null !== $page) {
            $select->limit($limit);
            $select->offset($limit * ($page - 1));
        }

        $select->group(['tag_id']);

        $adapter = new \Zend\Paginator\Adapter\DbSelect($select, $sql);
        $paginator = new \Zend\Paginator\Paginator($adapter);
        $paginator->setItemCountPerPage($limit);
        if (null !== $page) {
            $paginator->setCurrentPageNumber($page);
        }
        return $paginator;
    }
}
