<?php
/**
 * Copyright (C) 2019 Leipzig University Library
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 */

namespace fid_bbi\Helper;

use DOMDocument;
use VuFind\Date\Converter;
use Zend\Cache\Storage\StorageInterface;
use Zend\Config\Config;
use Zend\View\Helper\AbstractHelper;

/**
 * Class Rss Provides feed reader functionality
 * Mostly a copy from instance/fid_adlr,
 * * adds excerpt and date,
 * * returns objects instead of arrays
 * * provides __invoke
 * * alters constructor to include config and dateConverter
 *
 * @deprecated pull up to FID-module
 * @package fid_bbi\Helper
 */
class Rss extends AbstractHelper
{
    /**
     * @var Config
     */
    protected $rssConfig;

    /**
     * @var Converter
     */
    protected $dateConverter;

    /**
     * @var StorageInterface
     */
    protected $cache;

    /**
     * @var bool
     */
    protected $caching;

    /**
     * @var string
     */
    protected $cacheKey;

    /**
     * Rss constructor.
     * @param Config $rssConfig
     * @param Converter $dateConverter
     * @param StorageInterface $cacheManager
     */
    public function __construct(Config $rssConfig, Converter $dateConverter, StorageInterface $cache)
    {
        $this->rssConfig = $rssConfig;
        $this->dateConverter = $dateConverter;
        $this->feedDateFormat = $this->rssConfig->input_date_format ?? 'c';
        $this->prepareCache($cache);
    }

    protected function prepareCache(StorageInterface $cache)
    {
        $ttl = $this->rssConfig['cache_ttl'] ?? 0;
        if ($ttl == -1) {
            $this->caching = false;
        } else {
            $this->caching = true;
            $this->cache = $cache;
            $this->cache->getOptions()->setTtl($ttl);
            $this->cacheKey = md5(implode($this->rssConfig->toArray()));
        }
    }

    /**
     * Read cached feed
     *
     * @return bool|mixed cached feed or false if caching is inactive
     */
    protected function readCacheData()
    {
        if ($this->caching) {
            return $this->cache->getItem($this->cacheKey);
        }
        return false;
    }

    protected function writeCacheData($data)
    {
        if ($this->caching) {
            $this->cache->setItem($this->cacheKey,$data);
        }
    }

    /**
     * Read the feed
     *
     * @return array|bool|mixed
     */
    public function __invoke()
    {
        return $this->getFeed();
    }

    /**
     * get Rss feed data
     *
     * @return array|bool|mixed
     */
    protected function getFeed()
    {
        if ($cacheData = $this->readCacheData()) {
            return $cacheData;
        } else {
            $feed = $this->load($this->rssConfig);
            $this->writeCacheData($feed);
            return $feed;
        }
    }

    /**
     * @param array of rss origin data
     * @return array of rss feeds
     */
    public function load($rssOrigin)
    {
        $baseUrl    = $rssOrigin["base_url"] ?? "";
        $fallback   = $rssOrigin["fallback_url"] ?? "";
        $docPath    = $rssOrigin["xml_doc"] ?? "";

        $rssDoc = new DOMDocument();
        $feeds = array();
        try {
            if ($rssDoc->load($baseUrl . '/' . $docPath)) {
                $limit = $rssOrigin['limit'] ?? 3;
                $n = 0;

                foreach ($rssDoc->getElementsByTagName('item') as $node) {
                    $excerpt = $node->getElementsByTagName('description')->item(0)->nodeValue;
                    if (strlen($excerpt) > $this->rssConfig->excerpt_length) {
                        $excerpt = substr($excerpt, 0, $this->rssConfig->excerpt_length);
                        $lastSpace = strrpos($excerpt, ' ');
                        $excerpt = substr($excerpt, 0, $lastSpace);
                        if (isset($this->rssConfig['excerpt_suffix'])) {
                            $excerpt .= $this->rssConfig['excerpt_suffix'];
                        }
                    }
                    $date = $node->getElementsByTagName('pubDate')->item(0)->nodeValue;
                    $date = $this->dateConverter->convertToDisplayDate($this->feedDateFormat,$date);
                    $item = array(
                        'title' => $node->getElementsByTagName('title')->item(0)->nodeValue,
                        'link' => $node->getElementsByTagName('link')->item(0)->nodeValue,
                        'excerpt' => $excerpt,
                        'date' => $date,
                    );
                    array_push($feeds, (object)$item);

                    if (++$n >= $limit) {
                        break;
                    }
                }
            }
        } catch (\Exception $ex) {
            if ($fallback) {
                $item = array(
                    'title' => $fallback,
                    'link' => $fallback
                );
                array_push($feeds, $item);
            }
        }

        return $feeds;
    }
}
